"""cache.py

Part of the Tundra project.
(C) 2006 Rebuild Software
http://rebui.ld.net.au

This file defines containers for holding commonly-used resources.
"""

import globals as g

import sys, os

import pygame
from pygame.locals import *

import style


class StyleCache:
    def __init__(self):
        self.cache = {}

    def get(self, styleName):
        if not self.cache.has_key(styleName):
            self.cache[styleName] = style.importStyleFromXML(g.RESOURCES_PATH+"styles/"+styleName+".xml")
        return self.cache[styleName]

class FontCache:
    def __init__(self, name="font"):
        self.name = name
        self.cache = {}
    
    def get(self, letter):
        if not self.cache.has_key(letter):
            self.cache[letter] = pygame.image.load(g.RESOURCES_PATH+"pixmaps/"+self.name+"/"+repr(ord(letter)) + ".png")
        return self.cache[letter]
        
class CountdownCache:
    def __init__(self):
        self.cache = {}
    # TODO: replace get with __call__
    def get(self, number):
        if not self.cache.has_key(number):
            numberStr = str(number) # bomb numbers MUST be 5px*5px
            if len(numberStr) == 1: 
                self.cache[number] = pygame.image.load(g.RESOURCES_PATH+"pixmaps/count/bomb_"+str(number)+ ".png").convert_alpha()
            else:
                self.cache[number] = pygame.Surface([len(numberStr)*(6)-1,5],SWSURFACE|SRCALPHA).convert_alpha()
                index = 0
                for n in numberStr:
                    self.cache[number].blit(self.get(int(n)),(index*6,0))
                    index += 1

        return self.cache[number]


class SkillCache:
    def __init__(self):
        self.cache = {}
        self.dirtColour = map(int, g.FAVOURITE_COLOUR.split(","))
        #sys.path.insert(0,"skills/")
        return
    
    def get(self, skillName, lemming):
        if not self.cache.has_key(skillName):
            self._load(skillName)     
            
        if lemming.currentSkill == skillName:
            self.cache[skillName].update(lemming)
        else:
            self.cache[skillName].set(lemming)
        return

    def cleanUp(self, skillName, lemming):
        if not self.cache.has_key(skillName):
            self._load(skillName)     

        if hasattr(self.cache[skillName], "cleanUp"):
            self.cache[skillName].cleanUp(lemming)
        return     
    
    def check(self, skillName, attr):
        if not self.cache.has_key(skillName):
            self._load(skillName)     

        if attr in self.cache[skillName].attrs:
            return bool(self.cache[skillName].attrs[attr])
        
        return False
        
    def frames(self, skillName):
        if not self.cache.has_key(skillName):
            self._load(skillName)
        return self.cache[skillName].frames
    
    
    # NOTE: EVENTUALLY THIS WILL BE OBSOLETED IN FAVOUR OF CHANGING THE ENTIRE LEMMING PALETTE
    def updateDirt(self, colour):
        self.dirtColour = colour
        for i in self.cache.itervalues():
            if hasattr(i, "frames"):
                for j in i.frames.itervalues(): # left and right
                    for k in j[0]:
                        try:
                            k.set_palette_at(7, colour)
                        except pygame.error:
                            pass
                    

    # next section contains the hackladen methodus for precaching all sprites and rects
    # note how I've cleverly avoided doing work by using listslicing instead of regexes
    # to parse filenames :/
    # needless to say it won't work unless you use decrunch to produce the animation files
    def _load(self, skillName):
        self.cache[skillName] = __import__("skills."+skillName,[],[],["*"])
        if hasattr(self.cache[skillName], "frames"):
            dlist = os.listdir(g.RESOURCES_PATH + "pixmaps/lemmings/")            
            if(self.cache[skillName].frames.has_key(-1) and self.cache[skillName].frames.has_key(1)):
                lIndex = [m[:len(skillName)+2] for m in dlist].index(skillName+"_l")
                lSurface = pygame.image.load(g.RESOURCES_PATH + "pixmaps/lemmings/" + dlist[lIndex])
                lSurface.set_palette_at(7, self.dirtColour)
                lAnimHeight = int(dlist[lIndex][len(skillName)+4:][:-4])
                lAnimWidth = lSurface.get_width()
                self.cache[skillName].frames[-1][1] = Rect(0,0,lAnimWidth,lAnimHeight)
                for i in range(0,lSurface.get_height()/lAnimHeight):
                    srcRect = Rect(0, i*lAnimHeight, lAnimWidth, lAnimHeight)
                    self.cache[skillName].frames[-1][0].append(lSurface.subsurface(srcRect))
    
                if(self.cache[skillName].frames.has_key(-2)):
                    mlIndex = [m[:len(skillName)+3] for m in dlist].index(skillName+"_ml")
                    mlSurface = pygame.image.load(g.RESOURCES_PATH + "pixmaps/lemmings/" + dlist[mlIndex])
                    mlSurface = mlSurface.convert_alpha()
                    mlAnimHeight = int(dlist[mlIndex][len(skillName)+5:][:-4])
                    mlAnimWidth = mlSurface.get_width()
                    self.cache[skillName].frames[-2][1] = Rect(0,0,mlAnimWidth,mlAnimHeight)
                    for i in range(0,mlSurface.get_height()/mlAnimHeight):
                        srcRect = Rect(0, i*mlAnimHeight, mlAnimWidth, mlAnimHeight)
                        self.cache[skillName].frames[-2][0].append(mlSurface.subsurface(srcRect))
                    

                rIndex = [m[:len(skillName)+2] for m in dlist].index(skillName+"_r")
                rSurface = pygame.image.load(g.RESOURCES_PATH + "pixmaps/lemmings/" + dlist[rIndex])
                rSurface.set_palette_at(7, self.dirtColour)
                #rSurface = rSurface.convert_alpha()
                rAnimHeight = int(dlist[rIndex][len(skillName)+4:][:-4])
                rAnimWidth = rSurface.get_width()
                self.cache[skillName].frames[1][1] = Rect(0,0,rAnimWidth,rAnimHeight)
                for i in range(0,rSurface.get_height()/rAnimHeight):
                    srcRect = Rect(0, i*rAnimHeight, rAnimWidth, rAnimHeight)
                    self.cache[skillName].frames[1][0].append(rSurface.subsurface(srcRect))

                if(self.cache[skillName].frames.has_key(2)):
                    mrIndex = [m[:len(skillName)+3] for m in dlist].index(skillName+"_mr")
                    mrSurface = pygame.image.load(g.RESOURCES_PATH + "pixmaps/lemmings/" + dlist[mrIndex])
                    mrSurface = mrSurface.convert_alpha()
                    mrAnimHeight = int(dlist[mrIndex][len(skillName)+5:][:-4])
                    mrAnimWidth = mrSurface.get_width()
                    self.cache[skillName].frames[2][1] = Rect(0,0,mrAnimWidth,mrAnimHeight)
                    for i in range(0,mrSurface.get_height()/mrAnimHeight):
                        srcRect = Rect(0, i*mrAnimHeight, mrAnimWidth, mrAnimHeight)
                        self.cache[skillName].frames[2][0].append(mrSurface.subsurface(srcRect))

            elif(self.cache[skillName].frames.has_key(1)):
                index = [m[:len(skillName)] for m in dlist].index(skillName)
                surface = pygame.image.load(g.RESOURCES_PATH + "pixmaps/lemmings/" + dlist[index])
                surface.set_palette_at(7, self.dirtColour)
                #surface = surface.convert_alpha()
                animHeight = int(dlist[index][len(skillName)+2:][:-4])
                animWidth = surface.get_width()
                self.cache[skillName].frames[1][1] = Rect(0,0,animWidth,animHeight)
                for i in range(0,surface.get_height()/animHeight):
                    srcRect = Rect(0, i*animHeight, animWidth, animHeight)
                    self.cache[skillName].frames[1][0].append(surface.subsurface(srcRect))
                self.cache[skillName].frames[-1] = self.cache[skillName].frames[1]

                if(self.cache[skillName].frames.has_key(2)):
                    mIndex = [m[:len(skillName)+2] for m in dlist].index(skillName+"_m")
                    mSurface = pygame.image.load(g.RESOURCES_PATH + "pixmaps/lemmings/" + dlist[mIndex])
                    mSurface = mSurface.convert_alpha()
                    mAnimHeight = int(dlist[mIndex][len(skillName)+4:][:-4])
                    mAnimWidth = mSurface.get_width()
                    self.cache[skillName].frames[2][1] = Rect(0,0,mAnimWidth,mAnimHeight)
                    for i in range(0,mSurface.get_height()/mAnimHeight):
                        srcRect = Rect(0, i*mAnimHeight, mAnimWidth, mAnimHeight)
                        self.cache[skillName].frames[2][0].append(mSurface.subsurface(srcRect))
                    self.cache[skillName].frames[-2] = self.cache[skillName].frames[2]

        return
    
