"""style.py

Part of the Tundra project.
(C) 2006 Rebuild Software
http://rebui.ld.net.au

This file contains methods for loading and storing Tundra style collections.
"""


import os, sys
import gzip
import xml.sax
import xml.sax.writer
from xml.sax.handler import ContentHandler

import globals as g

try:
    import pygame
    from pygame.locals import *
except ImportError:
    raise ImportError, "You don't have Pygame installed."
try:
    assert pygame.image.get_extended()
except AssertionError:
    raise ImportError, "Your SDL_image does not support loading PNGs!"

class InterDef:
    def __init__(self):
        self.sourceImg = None
        self.frameHeight = 32
        self.interType = "harmless"
        self.activeRect = Rect((0,0),(32,32))
        self.frames = []

class TerraDef:
    def __init__(self):
        self.sourceImg = None
        self.subtractImg = None
        

class Style:
    def __init__(self):
        self.styleName = "A Brand New Look"
        self.author = "Ponds Institute"
        self.version = "1.0"
        self.favColour = printableToRgb(g.FAVOURITE_COLOUR)
        self.scale = 2
        self.interDefs = []
        self.terraDefs = []

    def exportXML(self, filename):
        f = open(filename,"w")
        xmlinfo = xml.sax.writer.XMLDoctypeInfo()
        xmlinfo.add_element_container("style")
        xmlinfo.add_element_container("interDefs")
        xmlinfo.add_element_container("terraDefs")
        xmlout = xml.sax.writer.PrettyPrinter( f , dtdinfo=xmlinfo )
        xmlout.startDocument()
        xmlout.startElement("style",{"styleName":self.styleName, "author":self.author, "version":self.version, "scale":str(self.scale)})
        xmlout.startElement("interDefs",{})
        # TODO: insert export commands for images
        xmlout.endElement("interDefs")
        xmlout.startElement("terraDefs",{})
        # TODO: insert export commands for images
        xmlout.endElement("terraDefs")
        xmlout.endElement("style")
        xmlout.endDocument()
        f.close()

class StyleHandler(ContentHandler):
    def __init__(self, filename):
        self.style = Style()
        self.filename = filename
        if self.filename[len(g.RESOURCES_PATH+"styles/"):] != g.RESOURCES_PATH+"styles/":
            self.stylePath = os.path.split(self.filename)[0]+"/"  
        else:
            self.stylePath = g.RESOURCES_PATH+"styles/"
        self.isInterDefs = False
        self.isTerraDefs = False
    
    def startElement(self, name, attrs):
        if name == "style":
            self.style.styleName = str(attrs.get("styleName",self.style.styleName))
            self.style.author =  str(attrs.get("author",self.style.author))
            self.style.version = str(attrs.get("version",self.style.version))
            self.style.scale = int(attrs.get("scale",self.style.scale))
            self.style.favColour = printableToRgb(attrs.get("favColour",g.FAVOURITE_COLOUR))

        elif name == "interDefs":
            self.isInterDefs = True
        elif (name == "interDef") & self.isInterDefs:
            self.style.interDefs.append(InterDef())
            self.style.interDefs[-1].sourceImg = pygame.image.load(str(self.stylePath+attrs.get("path","default/nointer.png"))).convert()
            self.style.interDefs[-1].frameHeight = int(attrs.get("frameHeight",32))
            self.style.interDefs[-1].interType = str(attrs.get("interType","harmless"))
            self.style.interDefs[-1].frames = []
            p = (int(attrs.get("rectX",0)), int(attrs.get("rectY",0)))
            s = (int(attrs.get("rectW",0)), int(attrs.get("rectH",0)))
            self.style.interDefs[-1].activeRect = Rect(p,s)
            frameList = pygame.image.load(str(self.stylePath+attrs.get("path","default/nointer.png"))).convert_alpha()
            for i in range(0,frameList.get_height(),self.style.interDefs[-1].frameHeight):
            	self.style.interDefs[-1].frames.append(frameList.subsurface(Rect(0, i,frameList.get_width(), self.style.interDefs[-1].frameHeight )))
        elif name == "terraDefs":
            self.isTerraDefs = True
        elif (name == "terraDef") & self.isTerraDefs:
            self.style.terraDefs.append(TerraDef())
            self.style.terraDefs[-1].sourceImg = pygame.image.load(str(self.stylePath+attrs.get("path","default/noterra.png"))).convert_alpha()
            self.style.terraDefs[-1].subtractImg = pygame.image.load(str(self.stylePath+attrs.get("path","default/noterra.png")))
            # TODO: terrible hack that requires an indexed image
            self.style.terraDefs[-1].subtractImg.set_palette( ((255,255,255),)+255*(g.DEFAULT_COLOURKEY[:3],) )
            self.style.terraDefs[-1].subtractImg.set_colorkey( 0 )
            self.style.terraDefs[-1].subtractImg = self.style.terraDefs[-1].subtractImg

    def endElement(self, name):
        if name == "interDefs":
            self.isInterDefs = True
        elif name == "terraDefs":
            self.isTerraDefs = False

def importStyleFromXML(filename):
    parser = xml.sax.make_parser()
    handler = StyleHandler(filename)
    parser.setContentHandler(handler)
    parser.parse(open(filename))
    return handler.style

def printableToRgb(printable):
    return map(int, printable.split(","))
