
import globals as g

import pygame
from pygame.locals import *

import os, sys; sys.path.insert(0, "..")

import font, level, dialog, game, player, mouse
from title import Title
from mainloop import MainLoop

import ser

from pgu import gui
from pgu.gui.const import *

class Tundra:
    def __init__(self, loopName="Title"):
        for d in ["styles", "levels", "music"]:
            try:
                os.makedirs(g.RESOURCES_PATH+d)
            except OSError:
                pass
        pygame.init()
        pygame.display.set_caption("Tundra v"+g.TUNDRA_VERSION)

        g.screen = pygame.display.set_mode((g.SCREEN_WIDTH,g.SCREEN_HEIGHT))
        pygame.display.set_icon(pygame.image.load(g.RESOURCES_PATH+"pixmaps/icon.png"))
    
        g.mouse = mouse.Mouse()
        g.mouse.changeCursor("hand")
    
        print "Welcome to Tundra v"+g.TUNDRA_VERSION+"!"
        print "http://rebui.ld.net.au/software/tundra"
        
        g.screen.fill((0,0,0))
        title = font.createLayout(g.msg("loader_preconf"), g.SCREEN_WIDTH, 36,0)
        g.screen.blit(title, (0,0))
        pygame.display.flip()

        g.clock = pygame.time.Clock()
        self.leaving = False
        styles = [f for f in os.listdir(g.RESOURCES_PATH+"styles") if not f.startswith(".")]
        levels = [f for f in os.listdir(g.RESOURCES_PATH+"levels") if not f.startswith(".")]

        if "default" in styles:
            styles.pop(styles.index("default"))

        needsResources = ((len(styles) == 0) or (len(levels) == 0))
        if os.path.isfile(g.CONFIG_FILE):
            try:
                player.loadLevelDB(g.CONFIG_FILE)
                updateScreenSize()
                if not needsResources:
                    self.switchLoops(loopName) # .tundra exists, plenty o' resources
                else:
                    self.switchLoops("Loader", jumpToDecrunch=True) # .tundra exists, no resources 
            except ser.NotCerealizerFileError:
                self.switchLoops("Loader") # .tundra is corrupt
        else:
            self.switchLoops("Loader") # .tundra doesn't exist
        
    def switchLoops(self, loopName, **args):
        self.looper = globals()[loopName](self, **args)
        
    def play(self):
        while not self.leaving:
            self.looper.loop()
        
class Loader:
    def __init__(self, parent, jumpToDecrunch=False):
        self.app = gui.App()
        self.app.connect(gui.QUIT,self.app.quit,None) 
        c = gui.Table(width=g.SCREEN_WIDTH,height=g.SCREEN_HEIGHT)
        self.app.init(c)
            
        d = dialog.LoaderDialog()
        d.connect(CHANGE, parent.switchLoops, "Title")
        if jumpToDecrunch:
            d.renderPage("decrunch_lite")
        d.open()

    def loop(self):
        g.screen.fill((0,0,0))
        for e in pygame.event.get():
            if e.type is QUIT: raise SystemExit
            self.app.event(e)
        self.app.paint(g.screen)
    
        pygame.display.flip()
        g.clock.tick(g.TITLE_FPS)


def updateScreenSize():
    flags = 0
    if g.ENABLE_FULLSCREEN:
        flags |= FULLSCREEN
    g.screen = pygame.display.set_mode((g.SCREEN_WIDTH,g.SCREEN_HEIGHT),flags)
    g.refresh()
    return
    


# look how small the actual game is! two lines of code
if __name__ == '__main__':
    t = Tundra()
    t.play()
